#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

#################################################################################
# Created       : Martin Foster
# Modified      : 14-Dec-2006
#################################################################################
#
# Vote - Javascript component for Ethereal Realms that will allow for a simpler
#        application of the voting and also designed to show results if the
#        user has called it before and voted.
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3-3456 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Socket;							# Socket library
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Param;						# Parameter control
use Ethereal::Template;						# Template management


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $param;							# Parameter list
my $tmpl;							# Template handler

my $statement;							# Database statement
my $res;							# Results list

my $name;							# Poll name
my $percent;							# Score
my $pixel      = 2;						# Pixel space to use
my $spacer;							# Spacer to user
my $question;							# Poll question
my $total;							# Total votes
my $vote;							# Vote has been made

my $list;							# List of outputs
my $output;							# What to show users

my $addr;							# Address
my $pked;							# Packed address

my @options;							# Options you can vote for

my %poll;							# Poll hash

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();

	# Connect and fetch
	$database->Connect($cgi);	

	# Parameter handling
	$param = new Ethereal::Param($database, $cgi);
	$param->GetParam();
	
	# Link with hash
	$database->GetHashPoll(\%poll);

	# Instance
	$tmpl = new Ethereal::Template(\%poll);
	

	# Display header
	print $cgi->header('text/javascript');

	# Poll Identifier
	# Exists
	if (defined($param->{'PID'}))
	{
		# Does the almighty concure?
		($name, $question) = $database->DataGetPoll($param->{'PID'});

		# You may pass
		if (defined($name))
		{
			# Pull address
			$addr = $cgi->remote_host();
				
			# Invalid IP address
			if ($addr !~ /\d{1,3}\.\d{1,3}\.\d{1,3}/)
			{
				# Retrieve address
				$pked = gethostbyname($addr);

				# Form into human readable
				$addr = inet_ntoa($pked);
			}


			# Has there been a vote before
			($vote) = $database->DataGetPollVote($param->{'PID'}, $addr);

			# Link to complete results
			my $lresults = $database->{'SYS'}{'LnkExtVote'} . '?PID=' . $param->{'PID'};

			# Vote has been done
			unless (defined($vote))
			{
				# List name
				$database->GetList(\@options, "SELECT OptionName
				  FROM PollOption
				  WHERE PollID=?
				  ORDER BY OptionID", $param->{'PID'});

				# Generate options
				my $woption  = $cgi->radio_group('PRES', \@options, $options[0], 'True');
				my $wvote    = $cgi->submit($poll{'TxtVote'});
				my $wfstart  = $cgi->start_form(-action=>$database->{'SYS'}{'LnkExtVote'}) 
				             . $cgi->hidden('PID', $param->{'PID'});
				my $wfend    = $cgi->end_form();
				  

				# Generate the template for use
				$output = $tmpl->Pass('TmplGenerate',
					MNAME      => $name,
					MQUESTION  => $question,
					WOPTIONS   => $woption,
					WVOTE      => $wvote,
					LRESULTS   => $lresults,
					WSTARTFORM => $wfstart,
					WENDFORM   => $wfend
				 );
			}

			# Display mini-results
			else
			{
				# Start display
				$database->Pull(\$statement, "SELECT
					  COUNT(*)     AS \"Votes\",
					  o.OptionName AS \"Name\"
					FROM PollVote p, PollOption o
					WHERE p.PollID=?
					  AND p.PollID=o.PollID
					  AND p.OptionID=o.OptionID
					GROUP BY o.OptionId, o.OptionName
					ORDER BY o.OptionId",

				 $param->{'PID'}
				);


				# Confirmation
				if ($res = $statement->fetchrow_hashref())
				{
					# Prep
					$list = '';

					# Final tally
					($total) = $database->DataGetPollTotal($param->{'PID'});

					# Loop and execute
					do
					{
						# Where are we
						$percent = int((($res->{'Votes'}-1)/$total)*100);
						$spacer  = $percent * $pixel;

						# Display
						$list .= $tmpl->Pass('TmplMiniResult',
							MOPTION  => $res->{'Name'},
							MPERCENT => $percent,
							MSPACER  => $spacer,
							MVOTES   => $res->{'Votes'}-1
						 );

					# Looping, looping, looping
					} while ($res = $statement->fetchrow_hashref());


					# Complete display
					$output = $tmpl->Pass('TmplMini',
						LRESULTS   => $lresults,
						MQUESTION  => $question,
						MTITLE     => $name,
						MTOTAL     => $total,
						LSTRESULTS => $list
					 );
				}

				# End query and whining
				$statement->finish();
			}


			# Safety
			$output =~ s/(\n|\r)//gs;
			$output =~ s/\s{2,}//gs;
			$output =~ s/'/\\'/gs;

			# Escape characters and display
			print "document.write('" . $output . "');";
		}

		else
		{
			# Through javascript
			print "document.write('<b>[ERROR: Invalid parameters]</b>');"
		}
	}
		
	# Sound error
	else
	{
		# Through javascript
		print "document.write('<b>[ERROR: Invalid parameters]</b>');";
	}	
