#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

################################################################################
# Created       : Martin Foster
# Modified      : 02-Jan-2007
################################################################################
#
# Located Tags - Finds no longer used tags
# Copyright (C) 2000-2007  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3-3456 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler

################################################################################
# Data Members
################################################################################

my $database;							# Database handle
my $res;							# Results list
my $statement;							# Defined query

my $count;							# Counter
my $file;							# File name
my $line;							# Line
my $master;							# Master configuration used
my $search;							# Search line captured
my $tags;							# Occurence of tags
my $type;							# Type to search in


my @types;							# Types to search for
my @files;							# File listing


################################################################################
# Program Area
################################################################################

	# Connect to Database
	$database = new Ethereal::Database();
	$database->Connect();


	# Determine method
	# To the point or wide scan
	if (defined($ARGV[0]))
	{
		# Push onto array and bypass testing
		push (@types, $ARGV[0]);
	}

	# Wide scans
	else
	{
		# Pull list of types
		$database->GetList(\@types, "SELECT TagType
			FROM Tag
			WHERE TagType NOT IN ('system','template','vcodes')
			GROUP BY TagType
			ORDER BY TagType");
	}
	
	# Fetch complete file listing
	DirSearch("../../main", \@files);
	DirSearch("../../mgmt", \@files);


	# Searching
	foreach $type (@types)
	{
		# Pull all tags
		$database->Pull(\$statement, "SELECT
			 TagName AS \"Name\"
			FROM Tag
			WHERE TagName NOT ILIKE 'TagLevel%'
			  AND TagName NOT IN ('TmplMain','TmplTitle','TmplSub','TmplAuth')
			  AND TagType='$type'
			GROUP BY TagName
			ORDER BY TagName");
	
		# Generate initial line
		$line = "DELETE FROM ethereal.Tag WHERE TagType='$type' AND TagName IN (";
	
		# Type
		$type = ucfirst($type);
		
		# Loop and discover
		while ($res = $statement->fetchrow_hashref())
		{
			# Main counter reset
			$count = 0;

			# Look through all files
			foreach $file (@files)
			{
				# Reset
				$tags   = 0;
				$master = 0;

				# Open the file for search
				open(SEARCH, $file);

				# Look through
				while ($search = <SEARCH>)
				{
					# Search for occurences
					$tags++   if ($search =~ /$res->{'Name'}/);
					$master++ if ($search =~ /GetHash$type/);
				}

				# Close the file
				close(SEARCH);


				# Increment
				# If master and tags are non-zero
				$count += $tags if (($tags > 0) && ($master > 0));
			}

			# Debugger display
			print $count . "\t - " . $type . " - " . $res->{'Name'} . "\n" ;
		
			# Tag that is no longer used
			$line .= "'" . $res->{'Name'} . "'," if ($count < 1); 
		}

		# Remove any trailing oddities
		chop($line);

		# Display
		print "\n\n" . $line . ");\n\n";
	}


#####################
# DirSearch
#
# Component necessary to look through directories and produce a proper file listing to
# return to the calling script.

sub DirSearch
{
	#####################
	# Data members

	my $dir      = shift;					# Directory name
	my $files    = shift;					# Reference to file listing

	my $file;						# File name
	my $sub;						# Subdirectory
	my $sep      = "/";					# Seperator

	my @locales;						# Additional directories


	#####################
	# Program area

	# Open Directory Handle 
	# Litmus test
	if (-d $dir)
	{
	 	opendir(DIR, $dir) or die("Error: $dir cannot be accessed!");	

		# Read through directory listing
		while ($file = readdir(DIR))
		{
			# Skip parent and current 
			next if ($file eq '.');
			next if ($file eq '..');			

			# Remove folders recursively 
			# Add to list
			if (-d "$dir$sep$file") 
			{
				# Pushes onto a list to look into later
				push(@locales, "$dir$sep$file");
			}

			# Append to list
			elsif ((-f "$dir$sep$file")
			 && (! -l "$dir$sep$file"))
			{
				# Push onto the file list
				push(@{$files}, "$dir$sep$file");
			}
		}	

		# Close handle
		closedir(DIR);

		# Call additional directories
		foreach $sub (@locales)
		{
			# Call subdirectory
			DirSearch($sub, $files);
		}
	}

	else
	{
		# Simply send a warning
		warn("Warning: $dir does not exist!");
	}
}
